/* Copyright (C) 2010 Stephane Rochoy
 * 
 * This file is part of Weapon Factory.
 * 
 * Weapon Factory is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Weapon Factory is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Weapon Factory. If not, see <http://www.gnu.org/licenses/>.
 */
#include "Core.hh"

#include "Exception.hh"

using namespace WeaponFactory;

Core::Core(Ogre::SceneNode*  cameraNode)
  : mIsRunning(false), 
    mMapView(new MapView(cameraNode))
{
}

Core::~Core() 
{
  if (mMapView) {
    delete mMapView;
    mMapView = 0;
  }
}

void
Core::addTime(Ogre::Real offset) 
{
  mMapView->addTime(offset);
  mEntityFactory.addTime(offset);
}

// Entities ///////////////////////////////////////////////////////////////////

Entity*
Core::createEntity(std::string type, std::string name) 
{
  return mEntityFactory.createEntity(type, name);
}

// Status ////////////////////////////////////////////////////////////////////

/** Tell if the game is still running by returning the state of the mIsRunning
 *  flag.
 */
bool
Core::isRunning(void) 
{
  return mIsRunning;
}

/** Set the state of the mIsRunning flag to enter or leave the game.
 */
void
Core::setRunning(bool state)
{
  mIsRunning = state;
}

// Camera /////////////////////////////////////////////////////////////////////

void 
Core::cameraDown(void) 
{
  mMapView->down();
}

void
Core::cameraLeft(void) 
{
  mMapView->left();
}

void 
Core::cameraRight(void) 
{
  mMapView->right();
}

void 
Core::cameraUp(void) 
{
  mMapView->up();
}

void
Core::moveTrooper(void)
{
  Entity* entity = mEntityFactory.getEntity("trooper");
  if (entity) {
    entity->queueMove(Ogre::Vector3(200, 0, 200));
  }
}
