/* Copyright (C) 2010 Stephane Rochoy
 * 
 * This file is part of Weapon Factory.
 * 
 * Weapon Factory is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Weapon Factory is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Weapon Factory. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef CORE_HH
#define CORE_HH
#include <Ogre.h>
#include "Entity.hh"
#include "EntityFactory.hh"
#include "MapView.hh"

namespace WeaponFactory {
  /** Hold the state of the current game.
   */
  class Core {
  public:
    Core(Ogre::SceneNode* cameraNode);
    ~Core();
    void addTime(Ogre::Real offset);

    // Entities:
    Entity* createEntity(std::string type, std::string name); 

    // Status:
    bool isRunning(void);
    void setRunning(bool state);

    // Camera:
    void cameraDown(void);
    void cameraLeft(void);
    void cameraRight(void);
    void cameraUp(void);

    void moveTrooper(void);

  protected:
    EntityFactory mEntityFactory; 
    bool          mIsRunning;
    MapView*      mMapView;
  };
}
#endif // CORE_HH
