/* Copyright (C) 2010 Stephane Rochoy
 * 
 * This file is part of Weapon Factory.
 * 
 * Weapon Factory is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Weapon Factory is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Weapon Factory. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef ENGINE_HH
#define ENGINE_HH
//#include <CEGUI.h>
//#include <OgreCEGUIRenderer.h>
#include <Ogre.h>

#include "FrameListener.hh"
#include "Entity.hh"
#include "InputDispatcher.hh"
#include "GameInputListener.hh"
#include "UserInterface.hh"

namespace WeaponFactory {
  /** The Weapon Factory game engine.
   *  The setup() method initialize the engine. The InputDispatcher is in charge
   *  of dispatching keyboard and mouse events to the appropriate InputListener
   *  according to the selected context. This events are captured each time a
   *  frame start being rendered through the FrameListener::frameStarted()
   *  method.
   */ 
  class Engine 
    : public Ogre::WindowEventListener {
  public:
    Engine();
    ~Engine();
    /** Initialize, run and cleanup the engine. 
     */
    void run(void);
  protected:
    /** Initialize the engine.
     */ 
    bool setup(void);
    void setupResources(void);
    void loadResources(void);
    bool configure(void);
    void createWindow(void);
    void chooseSceneManager(void);
    void createSceneNodes(void);
    void createCamera(void);
    void createViewports(void);
    void createScene(void);
    void createUserInterface(void);
    void createCore(void);

    /** Create the InputDispatcher that take in charge dispatching keyboard and
     *  mouse events to the appropriate InputListener.
     */
    void createInputSystem(void);

    /** Create the FrameListener that take in charge frame rendering as well as
     *  capturing keyboard and mouse events.
     */
    void createFrameListener(void);

    void createEntities(void);
    void createGround(void);

    void destroyScene(void);
    void destroyWindow(void);

    // Ogre::WindowEventListener interface ////////////////////////////////////

    bool windowClosing(Ogre::RenderWindow* window);

  protected:
    Ogre::Root*         mRoot;
    Ogre::String        mResourcePath;
    Ogre::Camera*       mCamera;
    Ogre::RenderWindow* mWindow;
    Ogre::SceneManager* mSceneMgr;


    OIS::InputManager* mInputManager;
    OIS::Keyboard*     mKeyboard;
    OIS::Mouse*        mMouse;
    InputDispatcher*   mInputDispatcher;
    /** The InputListener used in the "game" context. */
    GameInputListener* mGameInputListener; 

    FrameListener* mFrameListener;

    Core*   mCore;

    UserInterface* mUserInterface;
  };
}
#endif // ENGINE_HH
