/* Copyright (C) 2010 Stephane Rochoy
 * 
 * This file is part of Weapon Factory.
 * 
 * Weapon Factory is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Weapon Factory is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Weapon Factory. If not, see <http://www.gnu.org/licenses/>.
 */
#include "Exception.hh"
#include "MoveState.hh"
#include "Entity.hh"

using namespace WeaponFactory;

Entity::Entity(std::string type, std::string name)
{
  using namespace Ogre;

  SceneManager* sceneMgr
    = Root::getSingletonPtr()
      ->getSceneManager("main.scene-manager.weapon-factory");
  Ogre::Entity* oEntity
    = sceneMgr->createEntity(name + ".entity.weapon-factory", type + ".mesh");
  sceneMgr
    ->getSceneNode(name + ".node.weapon-factory")
    ->attachObject(oEntity);
  mEntity = oEntity;
  mMoveState = new MoveState(oEntity->getParentSceneNode());

  LogManager::getSingletonPtr()
    ->logMessage("Animations:");
  AnimationStateSet* animations = oEntity->getAllAnimationStates();
  AnimationStateIterator it = animations->getAnimationStateIterator();
  for (; it.hasMoreElements(); it.moveNext()) {
    AnimationState* as = it.peekNextValue();
    //printf("  %s\n", as->getAnimationName());
    LogManager::getSingletonPtr()
      ->logMessage("  " + as->getAnimationName());
  }

  mAnimationState = mEntity->getAnimationState("Idle");
  mAnimationState->setLoop(true);
  mAnimationState->setEnabled(true);

  LogManager::getSingletonPtr()				\
    ->logMessage("A new entity is born: " + mEntity->getName());
}

Entity::~Entity()
{
  delete mMoveState;
  mMoveState = 0;
}

void
Entity::addTime(Ogre::Real offset) 
{
  using namespace Ogre;

  if (mMoveState->isMoving()) {
    mMoveState->addTime(offset);
    if (!mMoveState->isMoving()) {
      mAnimationState = mEntity->getAnimationState("Idle");
      mAnimationState->setLoop(true);
      mAnimationState->setEnabled(true);
    }
  }
  mAnimationState->addTime(offset);
}

void
Entity::setPosition(Ogre::Vector3 position)
{
  mEntity->getParentSceneNode()->setPosition(position);
}

void
Entity::queueMove(Ogre::Vector3 hop)
{
  if (!mMoveState->isMoving()) {
    mAnimationState = mEntity->getAnimationState("Walk");
    mAnimationState->setLoop(true);
    mAnimationState->setEnabled(true);
  }
  mMoveState->queueMove(hop);
}

bool
Entity::isMoving(void)
{
  return mMoveState->isMoving();
}

void
Entity::cancelAllMoves(void)
{
  mMoveState->cancelAllMoves();
}
