/* Copyright (C) 2010 Stephane Rochoy
 * 
 * This file is part of Weapon Factory.
 * 
 * Weapon Factory is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Weapon Factory is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Weapon Factory. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef ENTITY_HH
#define ENTITY_HH
#include <Ogre.h>
#include "MoveState.hh"

namespace WeaponFactory {
  /** An entity is something that is animated, can move and have physical
   *  properties.
   *  Combat units and buildings are entities.
   *  \note Entities should be created using an EntityFactory.
   */
  class Entity {
  public:
    Entity(std::string type, std::string name);
    ~Entity();

    /** Add time to the current AnimationState of the entity.
     */
    void addTime(Ogre::Real offset);

    /** Set the position of the entity.
     */
    void setPosition(Ogre::Vector3 position);

    /** Add a new hop at the end of the move queue.
     */
    void queueMove(Ogre::Vector3 hop);

    /** Return true if the entity is moving.
     */
    bool isMoving(void);

    /** Forget about all currently queued moves and stop moving.
     */
    void cancelAllMoves(void);

  protected:
    Ogre::Entity*         mEntity;
    MoveState*            mMoveState;
    Ogre::AnimationState* mAnimationState;
  };
}
#endif // ENTITY_HH
