/* Copyright (C) 2010 Stephane Rochoy
 * 
 * This file is part of Weapon Factory.
 * 
 * Weapon Factory is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Weapon Factory is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Weapon Factory. If not, see <http://www.gnu.org/licenses/>.
 */
#include "EntityFactory.hh"

using namespace WeaponFactory;

EntityFactory::EntityFactory()
{
}

EntityFactory::~EntityFactory()
{
  Entity* entity;
  for (std::deque<Entity*>::iterator it = mEntities.begin();
       it != mEntities.end();
       it++) {
    entity = *it;
    delete entity;
  }
}

Entity*
EntityFactory::createEntity(std::string type, std::string name)
{
  Entity* entity = new Entity(type, name);
  mEntities.push_back(entity);
  mEntitiesByName[name] = entity;
  return entity;
}

Entity*
EntityFactory::getEntity(std::string name)
{
  return mEntitiesByName[name];
}

void
EntityFactory::addTime(Ogre::Real offset)
{
  Entity* entity;
  for (std::deque<Entity*>::iterator it = mEntities.begin();
       it != mEntities.end();
       it++) {
    entity = *it;
    entity->addTime(offset);
  }
}

