/* Copyright (C) 2010 Stephane Rochoy
 * 
 * This file is part of Weapon Factory.
 * 
 * Weapon Factory is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Weapon Factory is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Weapon Factory. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef ENTITYFACTORY_HH
#define ENTITYFACTORY_HH
#include <deque>
#include "Entity.hh"

namespace WeaponFactory {
  /** Manage entities.
   *  Used to create/destroy entities as well as to perform actions on alls or
   *  subsets of entities (e.g., addTime()).
   */
  class EntityFactory {
  public:
    EntityFactory();

    /** Destroy all the entities created by this factory as well as the factory
     *  itself.
     */
    ~EntityFactory();

    /** Create a new entity.
     */
    Entity* createEntity(std::string type, std::string name);

    Entity* getEntity(std::string name);

    /** Add time to all entities created by this factory.
     */ 
    void addTime(Ogre::Real offset);

  protected:
    /** The list of entities created by this factory. 
     */
    std::deque<Entity*>            mEntities;
    std::map<std::string, Entity*> mEntitiesByName;
  };
}
#endif // ENTITYFACTORY_HH
