/* Copyright (C) 2010 Stephane Rochoy
 * 
 * This file is part of Weapon Factory.
 * 
 * Weapon Factory is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Weapon Factory is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Weapon Factory. If not, see <http://www.gnu.org/licenses/>.
 */
#include <fstream>
#include "Exception.hh"
#include "InputDispatcher.hh"

using namespace WeaponFactory;

InputDispatcher::InputDispatcher(OIS::Keyboard* keyboard, OIS::Mouse* mouse)
  : mKeyboard(keyboard), mMouse(mouse)
{
}

void 
InputDispatcher::pushContext(const std::string context)
{
  InputListener* newListener = mListeners[context];

  setListener(newListener);
  mContextStack.push(context);
}

std::string
InputDispatcher::popContext(void)
{
  std::string oldContext = mContextStack.top();
  mContextStack.pop();
  std::string newContext = mContextStack.top();
  InputListener* newListener = mListeners[newContext];

  setListener(newListener);
  return oldContext;
}

void
InputDispatcher::setListener(InputListener* newListener)
{
  mKeyboard->setEventCallback(newListener);
  mMouse->setEventCallback(newListener);
  mCurrentListener = newListener;
}

void
InputDispatcher::bindListener(InputListener* listener, const std::string context)
{
  mListeners[context] = listener;
}

void
InputDispatcher::unbindListener(std::string context)
{
}

void
InputDispatcher::capture(void)
{
  mKeyboard->capture();
  mMouse->capture();
}
