/* Copyright (C) 2010 Stephane Rochoy
 * 
 * This file is part of Weapon Factory.
 * 
 * Weapon Factory is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Weapon Factory is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Weapon Factory. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef INPUTDISPATCHER_HH
#define INPUTDISPATCHER_HH
#include <map>
#include <stack>
//#include <Ogre.h>
#include "InputListener.hh"

namespace WeaponFactory {
  /** Dispatch keyboard and mouse input events to the corresponding
   *  InputListener according to the current context.
   */
  class InputDispatcher {
  public:
    InputDispatcher(OIS::Keyboard* keyboard, OIS::Mouse* mouse);

    /** Select a new context and remember the previous one.
     *  The new context must exist. 
     */
    void pushContext(const std::string context);

    /** Restore the previous context.
     *  Throw an exception if no context was previously selected.
     */
    std::string popContext(void);

    /** Bind an InputListener to a context.
     *  Throw an exception if a listener was already bound to the context. Only
     *  one listener can be bound to a given context. The context is created if
     *  it didn't already exist. 
     */
    void bindListener(InputListener* listener, const std::string context);

    /** Remove listener binding for the current context.
     *  Throw an exception if no listener where bound to the current context.
     */
    void unbindListener(std::string context);

    /** Capture input and dispatch it to the listener bound to the currently
     *  selected context.
     */
    void capture(void);

  protected:
    void setListener(InputListener* newListener);

    OIS::Keyboard*                              mKeyboard;
    OIS::Mouse*                                 mMouse;
    std::map<const std::string, InputListener*> mListeners;
    std::stack<std::string>                     mContextStack;
    InputListener*                              mCurrentListener;
  };
}

#endif // INPUTDISPATCHER_HH
