/* Copyright (C) 2010 Stephane Rochoy
 * 
 * This file is part of Weapon Factory.
 * 
 * Weapon Factory is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Weapon Factory is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Weapon Factory. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef INPUTLISTENER_HH
#define INPUTLISTENER_HH
#include <CEGUI.h>
#define OIS_DYNAMIC_LIB
#include <OIS.h>
#include "Functor.hh"

namespace WeaponFactory 
{
  /** Handle keyboard and mouse input events.
   *  This class is also responsible for loading key bindings.
   */
  class InputListener 
    : public OIS::KeyListener,
      public OIS::MouseListener {
  public:
    InputListener(void* opaqueData);

    void bindKey(OIS::KeyCode key, const std::string actionName);
    void loadKeyBindings(const std::string fname);
    void registerAction(const std::string name, Functor* functor);
    void triggerAction(const std::string name);

  protected:
    // OIS::KeyListener interface:
    bool keyPressed(const OIS::KeyEvent &e);
    bool keyReleased(const OIS::KeyEvent &e);

    // OIS::MouseListener interface:
    bool mouseMoved(const OIS::MouseEvent &arg);
    bool mousePressed(const OIS::MouseEvent &arg, OIS::MouseButtonID id);
    bool mouseReleased(const OIS::MouseEvent &arg, OIS::MouseButtonID id);

    CEGUI::MouseButton convertMouseButton(OIS::MouseButtonID buttonID);

  protected:
    void*                                  mOpaqueData;
    std::map<const std::string,  Functor*> mActions;
    std::map<const OIS::KeyCode, Functor*> mKeyBindings;
  };
}
#endif // INPUTLISTENER_HH
