/* Copyright (C) 2010 Stephane Rochoy
 * 
 * This file is part of Weapon Factory.
 * 
 * Weapon Factory is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Weapon Factory is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Weapon Factory. If not, see <http://www.gnu.org/licenses/>.
 */
#include "Exception.hh"
#include "MoveState.hh"

using namespace WeaponFactory;

MoveState::MoveState(Ogre::SceneNode* node) 
  : mCurrentPosition(node->getPosition()),
    mDestination(node->getPosition()),
    mDirection(Ogre::Vector3::ZERO),
    mDistance(0.0f),
    mNode(node),
    mSpeed(23.0f) 
{
  using namespace Ogre;

  if (!node) {
    throw WeaponFactory::Exception();
  }
  LogManager::getSingletonPtr()
    ->logMessage("MoveState attached to node " + node->getName());
}

void
MoveState::setupNextHop(void)
{
  using namespace Ogre;

  if (mHops.size() > 0) {
    printf("mCurrentPosition = (%f, %f, %f)\n", mCurrentPosition.x, mCurrentPosition.y, mCurrentPosition.z);
    mDestination = *(mHops.begin());
    printf("mDestination = (%f, %f, %f)\n", mDestination.x, mDestination.y, mDestination.z);
    mHops.pop_front();
    mDirection = mDestination - mCurrentPosition; 
    mDistance  = mDirection.length();
    printf("mDistance = %f\n", mDistance);
    mDirection.normalise();
    printf("mDirection = (%f, %f, %f)\n", mDirection.x, mDirection.y, mDirection.z);
    mNode->setDirection(mDirection, SceneNode::TS_LOCAL, Vector3::UNIT_X); 
  } else {
    mDirection = Vector3::ZERO;
    mDistance  = 0.0f;
  }
}

void
MoveState::addTime(Ogre::Real offset) 
{
  using namespace Ogre;

  //printf("mSpeed = %f, offset = %f\n", mSpeed, offset);
  Real move = mSpeed * offset;
  //printf("move = %f\n", move);
  mDistance -= move;
  /*
  printf("mDistance = %f\n", mDistance);
  printf("mDirection = (%f, %f, %f)\n", mDirection.x, mDirection.y, mDirection.z);
  */
  if (mDistance <= 0.0f) {
    mNode->setPosition(mDestination);
    setupNextHop();
  } else {
    mNode->translate(mDirection * move);
  }
}

bool
MoveState::isMoving(void) 
{
  //return !mHops.empty();
  return mDistance > 0.0f;
}

void
MoveState::queueMove(Ogre::Vector3 hop) 
{
  mHops.push_back(hop);
  Ogre::LogManager::getSingletonPtr()
    ->logMessage("New hop queued");
  if (!isMoving()) {
    setupNextHop();
  }
}

void
MoveState::cancelAllMoves(void) 
{
  mHops.clear();
}
