/* Copyright (C) 2010 Stephane Rochoy
 * 
 * This file is part of Weapon Factory.
 * 
 * Weapon Factory is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Weapon Factory is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Weapon Factory. If not, see <http://www.gnu.org/licenses/>.
 */
#include "ShellWindow.hh"
#include <Ogre.h>

using namespace WeaponFactory;

ShellWindow::ShellWindow(CEGUI::WindowManager* wmgr, Shell* shell)
  : mShell(shell)
{
  using namespace CEGUI;

  mSheet = wmgr->createWindow("DefaultWindow", "CEGUIDemo/Sheet");

  // mOutput = shell.multiline-editbox.weapon-factory
  mOutput = wmgr->createWindow("TaharezLook/MultiLineEditbox", 
                               "shell.multiline-editbox.weapon-factory");
  mOutput->setText("Hello, world!");
  mOutput->setSize(UVector2(UDim(1.0, 0), UDim(0.9, 0)));
  mOutput->setEnabled(false);
  mSheet->addChildWindow(mOutput);

  // mInput = shell.editbox.weapon-factory
  mInput = wmgr->createWindow("TaharezLook/Editbox", "shell.editbox.weapon-factory");
  //mInput->setText("Hello from editBox!");
  mInput->setPosition(UVector2(UDim(0, 0), UDim(0.9, 0)));
  mInput->setSize(UVector2(UDim(1.0, 0), UDim(0.1, 0)));
  mInput->subscribeEvent(Editbox::EventTextAccepted, 
                         Event::Subscriber(&ShellWindow::onTextAccepted, this));

  mSheet->addChildWindow(mInput);
  mSheet->setVisible(false);
}

bool
ShellWindow::onTextAccepted(const CEGUI::EventArgs& args)
{
  using namespace CEGUI;

  const WindowEventArgs& we = 
        static_cast<const WindowEventArgs&>(args);
  String text = we.window->getText();

  Ogre::LogManager::getSingletonPtr()
    ->logMessage("TextAccepted event triggered:");
  Ogre::LogManager::getSingletonPtr()
    ->logMessage(text.c_str());

  return true;
}

CEGUI::Window*
ShellWindow::getWindow(void)
{
  return mSheet;
}

bool
ShellWindow::isVisible()
{
  return mSheet->isVisible();
}

void
ShellWindow::setVisible(bool state)
{
  mSheet->setVisible(state);
}

bool 
ShellWindow::toggleVisible(void)
{
  this->setVisible(!this->isVisible());
  return this->isVisible();
}
