/* Copyright (C) 2010 Stephane Rochoy
 * 
 * This file is part of Weapon Factory.
 * 
 * Weapon Factory is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Weapon Factory is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Weapon Factory. If not, see <http://www.gnu.org/licenses/>.
 */
#include "UserInterface.hh"

using namespace WeaponFactory;

UserInterface::UserInterface(Ogre::RenderWindow* window, Ogre::SceneManager* sceneMgr, Core* core)
  : mCore(core)
{
  using namespace CEGUI;
  using namespace Ogre;

  // Initialize CEGUI.
  LogManager::getSingletonPtr()
    ->logMessage("Initializing CEGUI...");
  mRenderer = new CEGUI::OgreCEGUIRenderer(window);
  mRenderer->setTargetSceneManager(sceneMgr);
  new CEGUI::System(mRenderer);
  CEGUI::Imageset::setDefaultResourceGroup("Imagesets");
  CEGUI::Font::setDefaultResourceGroup("Fonts");
  CEGUI::Scheme::setDefaultResourceGroup("Schemes");
  CEGUI::WidgetLookManager::setDefaultResourceGroup("LookNFeel");
  CEGUI::WindowManager::setDefaultResourceGroup("Layouts");
  CEGUI::SchemeManager::getSingleton()
    .loadScheme("TaharezLook.scheme");
  CEGUI::System::getSingleton()
    .setDefaultMouseCursor("TaharezLook", "MouseArrow");
  CEGUI::MouseCursor::getSingleton()
    .setImage(CEGUI::System::getSingleton().getDefaultMouseCursor());

  // Build the window manager and the root window.
  CEGUI::WindowManager& wmgr = CEGUI::WindowManager::getSingleton();
  Window* rootWindow = wmgr.createWindow("DefaultWindow", "WeaponFactory/Root");
  CEGUI::System::getSingleton().setGUISheet(rootWindow);

  CEGUI::Font* font;
  if(CEGUI::FontManager::getSingleton().isFontPresent("Liberation")) {
    font = CEGUI::FontManager::getSingleton().getFont("Liberation");
  } else {
    font 
      = CEGUI::FontManager::getSingleton().createFont("Liberation.font");
  }

  // Build the main menu.
  // Build the menu bar.
  Menubar* menuBar 
    = static_cast<Menubar*>(wmgr.createWindow("TaharezLook/Menubar", 
	                                      "root/menubar"));
  UDim bar_bottom(0, font->getLineSpacing(2));
  menuBar->setArea(UDim(0, 0), UDim(0, 0),UDim(1, 0), bar_bottom);
  menuBar->setSize(UVector2(UDim(1.0f, 0.0f), UDim(0.05f, 0.0f)));
  menuBar->setAlwaysOnTop(true);
  rootWindow->addChildWindow(menuBar);

  // Build the "File" menu item.
  MenuItem* menuFile 
    = static_cast<MenuItem*>(wmgr.createWindow("TaharezLook/MenuItem", 
	                                       "root/menubar/file"));
  menuFile->setText("File");
  menuBar->addChildWindow(menuFile); 

  // create file menu popup window
  PopupMenu* popupMenuFile 
    = static_cast<PopupMenu*>(wmgr.createWindow("TaharezLook/PopupMenu", 
	                                        "root/menubar/file/popup"));
  menuFile->addChildWindow(popupMenuFile); 

  // create the "Toggle Console" menuitem
  MenuItem* menuItem;
  menuItem = static_cast<MenuItem*>(wmgr.createWindow("TaharezLook/MenuItem", "root/menubar/file/popup/toggle-console"));
  menuItem->setText("Toggle Console");
  menuItem->subscribeEvent(MenuItem::EventClicked, Event::Subscriber(&UserInterface::onToggleConsole, this)); 
  popupMenuFile->addChildWindow(menuItem);

  // create the "Exit" menuitem
  menuItem = static_cast<MenuItem*>(wmgr.createWindow("TaharezLook/MenuItem", "root/menubar/file/popup/exit"));
  menuItem->setText("Exit");
  menuItem->subscribeEvent(MenuItem::EventClicked, Event::Subscriber(&UserInterface::onExit, this)); 
  popupMenuFile->addChildWindow(menuItem);
}

bool UserInterface::onToggleConsole(const CEGUI::EventArgs& args)
{
  printf("Event: onToggleConsole\n");
}

bool UserInterface::onExit(const CEGUI::EventArgs& args)
{
  mCore->setRunning(false);
}
