/* Copyright (C) 2010 Stephane Rochoy
 * 
 * This file is part of Weapon Factory.
 * 
 * Weapon Factory is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Weapon Factory is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Weapon Factory. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef WEAPON_HH
#define WEAPON_HH
#include <deque>
#include "Entity.hh"

namespace WeaponFactory {
  /** A Weapon.
   *  A Weapon is something that acquire a target entity and shoot at it while
   *  it stay in range.
   */
  class Weapon
  {
  public:
    /** Return the entity that is currently targeted.
     *  Throw an exception if no entity is currently targeted.
     *  \return \e Entity*
     *    Return the entity that is currently targeted. 
     */
    Entity* getTarget(void);

    /** Target an entity.
     *  Set a new target for the weapon. Throw an exception if the entity is not
     *  in range.
     */
    void setTarget(Entity* entity);

    /** Tell if the weapon target an entity.
     *  \return \e bool
     *    Return true if an entity is currently targeted by the weapon.
     */
    bool hasTarget(void);

    /** Select a target and remove it from a list.
     *  \return \e Entity*
     *    The selected target.
     */
    Entity* acquireTarget(std::deque<Entity*>::iterator* entities);
  };
}

#endif // WEAPON_HH
