/* Copyright (C) 2010 Stephane Rochoy
 * 
 * This file is part of Weapon Factory.
 * 
 * Weapon Factory is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Weapon Factory is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Weapon Factory. If not, see <http://www.gnu.org/licenses/>.
 */
/** \mainpage
 *  Weapon Factory is a 3D realtime strategy game focused on tactic and strategy
 *  instead of the APM (Actions Per Minute) capabilities of the player. It
 *  feature a lot of improvements in the user interface in order to help plan
 *  and commands the units.
 *
 *  \section Description
 *  Before the game start, the players must agree on the amount of credits
 *  they'll begin with. They can spend this credits to buy units before the game
 *  start. They even can buy modules if they can attach each of them to a
 *  Carrier.
 *  Once all players finished there initial investements, the game start. Each
 *  player start at a random start location on the map.
 *  To win, a player must destroy every units and buildings of all its
 *  opponents.
 *
 *  \section Resources
 *
 *  \par Money
 *  Measured in Credits. Produced by \e banks. Required to build everything. The
 *  player can only use the money that reach one of its <em>core factories</em>
 *  (see \e Drone).
 *
 *  \par Energy
 *  Measured in Watts. Produced by <em>Energy Nodes</em>. Required by \e buildings
 *  and some other special entities (e.g., Grosse Bertha). 
 *
 *  \par Supply
 *  Measured in Heads. Produced by <em>Supply Depots</em>. Required to produce units.
 *
 *  \section Terminology
 *
 *  \par Add-on
 *  Building that can be plugged to another building to provide some additional
 *  capabability (e.g., more production queues).
 *
 *  \par Air Factory
 *  \e Factory that produce air units.
 *
 *  \par Bank
 *  Neutral building that provide \e drones with money.
 *
 *  \par Building
 *  Entity that cannot move by itself and must be near an 
 *  <em>Energy Node</em> to work.
 *
 *  \par Carrier
 *  Air unit that can be used to carry units and \e modules.
 *
 *  \par Core Factory
 *  \e Factory that produce \e drones. Credits must be bring to a <em>Core
 *  Factory</em> to be earned by the player. 
 *
 *  \par Drone
 *  Unit that get credits at \e banks and bring them to a <em>Core Factory</em>.
 *
 *  \par Energy Node
 *  \e Module that produce a fixed and constant amound of energy and allow buildings
 *  to be built arround it.
 *
 *  \par Engineering Dock
 *  Module that provide units with new ammos and can be used to repair units and
 *  replace their weapons. E.g., you can replace the heavy machine gun of your
 *  panzer with a rocket launcher from this module. You can also reload your
 *  unit with 5 mines from this module.
 *
 *  \par Factory
 *  Building that can morph into an heavy combat unit. When in unit form, cannot
 *  produce anything. 
 *
 *  \par Ground Factory
 *  \e Factory that produce ground units.
 *
 *  \par Module
 *  \e Building that can be moved using a \e Carrier.
 *
 *  \par Sensor
 *  \e Module, \e building or weapon that perform detection.
 *
 *  \par Shield Module
 *  \e Module that produce a protection dome (an AT Field ;).
 *
 *  \par Special Factory
 *  \e Factory that produce special units.
 *
 *  \par Supply Depot
 *  \e Building that produce supply.
 *
 *  \par Weapon Slot
 *  Each unit can have one or more weapon slots. Each slot can be filled with a
 *  weapon. The player can change the weapon plugged on a weapon slot of a unit
 *  using the <em>Engineering Dock</em>.
 */

/** \namespace WeaponFactory
 *  All entities belonging to the Weapon Factory game engine are in this
 *  namespace. 
 */

/** \file weapon-factory.cc
 *  Entry point of the game. Contains the main() function.
 */
#include "Engine.hh"

using namespace WeaponFactory;

/** Create an instance of the game engine and run it.
 */
int
main(int argc, char *argv[]) {
  Engine wf;

  try {
    wf.run();
  } catch (Ogre::Exception& e) {
    fprintf(stderr, "An exception has occurred: %s\n",
	    e.what());
  }  
  return 0;
}
